local addon = LibStub("AceAddon-3.0"):NewAddon("Defailure", "AceEvent-3.0", "AceTimer-3.0")
_G.Defailure = addon
local fail = LibStub("LibFail-1.0")
local LB = LibStub("LibBabble-Boss-3.0")
local THE_LICH_KING = LB["The Lich King"]
local _, _, DEFILE_ICON = GetSpellInfo(73708)
local DEFILE_LINK = GetSpellLink(73708)

local function iterateRaidMembers()
	local i = 0
	local n = GetNumRaidMembers()
	return function()
			i = i + 1
			if i <= n then
				local unit = "raid" .. i
				local name = UnitName(unit)
				return unit, name
			end
		end
end

local failstat
local dbDefaults = {
	profile = {
		output = "self",
	},
	faction = {},
}
function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("DefailureDB", dbDefaults, "Default")
	local failstatAll = self.db.faction
	local today = date("%y-%m-%d")
	failstat = failstatAll[today]
	if not failstat then
		failstat = {}
		failstatAll[today] = failstat
	end
end

function addon:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:UpdateConfig()
end

local targetPollTimer
local failed = false
local currentDefileTarget

function addon:COMBAT_LOG_EVENT_UNFILTERED(self, _, timestamp,event,sourceGUID,sourceName, sourceFlags, destGUID, destName, destFlags, spellId)
	if event == "SPELL_CAST_START" and spellId == 72762 then
		targetPollTimer = self:ScheduleRepeatingTimer("FindDefileTarget", 0.2)
	end
end

function addon:FindDefileTarget(self)
	for unitId in iterateRaidMembers() do
		local target = UnitName(unitId .. "target")
		if target == THE_LICH_KING then
			local ttarget = UnitName(unitId .. "targettarget")
			if ttarget then
				currentDefileTarget = ttarget
				failed = false
				self:CancelTimer(targetPollTimer, true)
			end
			return
		end
	end
end

local function reset()
	currentDefileTarget = nil
	failed = false
end

function addon:Broadcast(message)
	if self.db.profile.output == "self" then
		print(message)
	else
		SendChatMessage(message, self.db.profile.output)
	end
end

local combatStat = {}

local function onDefile(event, who, failtype)
	if failed then return end
	if who == currentDefileTarget then
		currentDefileTarget = nil
		return
	end
	addon:ScheduleTimer(reset, 10)

	local statEntry = failstat[who] or 0
	failstat[who] = statEntry + 1
	local entry = combatStat[who] or 0
	combatStat[who] = entry + 1

	failed = true
	local message = (">>>>>>>>> %s messed up %s"):format(who, DEFILE_LINK)

	addon:Broadcast(message, "RAID")
end
fail.RegisterCallback("Defailure", "Fail_TheLichKing_Defile", onDefile)

local function onWipe()
	for k,v in pairs(combatStat) do
		addon:Broadcast(("%s failed %dx at Defile"):format(k,v), "RAID")
	end
	wipe(combatStat)
end

local function checkForWipe()
    for unit in iterateRaidMembers() do
		if unit and UnitAffectingCombat(unit) then
			return false
		end
    end
    return true
end

local wipeCheckTimer
function addon:PLAYER_REGEN_DISABLED()
	wipeCheckTimer = self:ScheduleRepeatingTimer("KeepCheckingForWipe", 1)
end

function addon:KeepCheckingForWipe()
	if checkForWipe() then
		onWipe()
		self:CancelTimer(wipeCheckTimer, true)
	end
end

local menu = {}
local popupFrame = CreateFrame("Frame", "DefailureMenu", UIParent, "UIDropDownMenuTemplate")
local outputOptions = {
	RAID = RAID,
	PARTY = PARTY,
	self = "self",
}

for k,v in pairs(outputOptions) do
	table.insert(menu, {
		text = v,
		tooltipTitle = v,
		tooltipText = v,
		func = function()
				addon.db.profile.output = k
				addon:UpdateConfig()
			end,
	})
end

local ldbdata = {
	type = "data source",
	icon = DEFILE_ICON,
	OnClick = function(self, msg)
			if msg == "LeftButton" then
				addon:Broadcast("Defailure - Stats: ")
				for k,v in pairs(failstat) do
					if UnitExists(k) then
						addon:Broadcast(("%s failed %dx"):format(k,v))
					end
				end
			else
				EasyMenu(menu, popupFrame, self, 20, 4, "MENU")
			end
		end,
	label = "Defailure",
}

function addon:UpdateConfig()
	ldbdata.text = _G[self.db.profile.output] or self.db.profile.output
end

LibStub("LibDataBroker-1.1"):NewDataObject("Defailure", ldbdata)
